@@ -440,8 +440,8 @@ def is_valid_geom(geom):
     bool: True if object is a valid GeoJSON geometry type
     """
 
-    geom_types = {'Point', 'MultiPoint', 'LineString', 'MultiLineString',
-                  'Polygon', 'MultiPolygon'}
+    geom_types = {'Point', 'MultiPoint', 'LineString', 'LinearRing',
+                  'MultiLineString', 'Polygon', 'MultiPolygon'}
 
     if 'type' not in geom:
         return False
@@ -473,6 +473,11 @@ def is_valid_geom(geom):
             # a coordinate
             return len(coords) >= 2 and len(coords[0]) >= 2
 
+        if geom_type == 'LinearRing':
+            # Rings must have at least 4 coordinates and at least x, y for
+            # a coordinate
+            return len(coords) >= 4 and len(coords[0]) >= 2
+
         if geom_type == 'MultiLineString':
             # Multi lines must have at least one LineString
             return (len(coords) > 0 and len(coords[0]) >= 2 and
