@@ -8,7 +8,7 @@ from affine import Affine
 
 import rasterio
 from rasterio.enums import MergeAlg
-from rasterio.errors import WindowError, RasterioDeprecationWarning
+from rasterio.errors import WindowError
 from rasterio.features import (
     bounds, geometry_mask, geometry_window, is_valid_geom, rasterize, sieve,
     shapes)
@@ -33,6 +33,12 @@ def test_bounds_line():
     assert bounds(MockGeoInterface(g)) == (0, 0, 10, 10)
 
 
+def test_bounds_ring():
+    g = {'type': 'LinearRing', 'coordinates': [[0, 0], [10, 10], [10, 0]]}
+    assert bounds(g) == (0, 0, 10, 10)
+    assert bounds(MockGeoInterface(g)) == (0, 0, 10, 10)
+
+
 def test_bounds_polygon():
     g = {'type': 'Polygon', 'coordinates': [[[0, 0], [10, 10], [10, 0]]]}
     assert bounds(g) == (0, 0, 10, 10)
@@ -91,6 +97,7 @@ def test_geometry_mask_invert(basic_geometry, basic_image_2x2):
         )
     )
 
+
 def test_geometry_invalid_geom():
     """An invalid geometry should fail"""
 
@@ -298,6 +305,24 @@ def test_is_valid_geom_polygon(geojson_polygon):
     assert not is_valid_geom(geom)
 
 
+def test_is_valid_geom_ring(geojson_polygon):
+    """Properly formed GeoJSON LinearRing is valid"""
+    geojson_ring = deepcopy(geojson_polygon)
+    geojson_ring['type'] = 'LinearRing'
+    # take first ring from polygon as sample
+    geojson_ring['coordinates'] = geojson_ring['coordinates'][0]
+    assert is_valid_geom(geojson_ring)
+
+    # Empty iterables are invalid
+    geom = deepcopy(geojson_ring)
+    geom['coordinates'] = []
+    assert not is_valid_geom(geom)
+
+    geom = deepcopy(geojson_ring)
+    geom['coordinates'] = [[]]
+    assert not is_valid_geom(geom)
+
+
 def test_is_valid_geom_multipolygon(geojson_multipolygon):
     """Properly formed GeoJSON MultiPolygon is valid"""
 
@@ -595,6 +620,7 @@ def test_rasterize_all_touched(basic_geometry, basic_image):
         )
     )
 
+
 def test_rasterize_merge_alg_add(basic_geometry, basic_image_2x2x2):
     """
     Rasterizing two times the basic_geometry with the "add" merging
