@@ -3,12 +3,15 @@ import re
 import subprocess
 import sys
 
+import affine
 import numpy as np
 import pytest
 
 import rasterio
-from rasterio.errors import RasterioIOError
-from rasterio._io import BAD_WRITE_DRIVERS
+from rasterio.drivers import blacklist
+from rasterio.env import Env
+from rasterio.errors import RasterioIOError, DriverRegistrationError
+from rasterio._base import driver_supports_mode
 
 
 logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)
@@ -30,6 +33,16 @@ def test_validate_dtype_str(tmpdir):
             dtype='Int16')
 
 
+def test_validate_dtype_int8(tmpdir, basic_image):
+    """Raise TypeError if dtype is unsupported by GDAL."""
+    name = str(tmpdir.join('int8.tif'))
+    basic_image_int8 = basic_image.astype('int8')
+    height, width = basic_image_int8.shape
+    with pytest.raises(TypeError):
+        rasterio.open(name, 'w', driver='GTiff', width=width, height=height,
+                      count=1, dtype=basic_image_int8.dtype)
+
+
 def test_validate_count_None(tmpdir):
     name = str(tmpdir.join("lol.tif"))
     with pytest.raises(TypeError):
@@ -46,6 +59,7 @@ def test_no_crs(tmpdir):
             dtype=rasterio.uint8) as dst:
         dst.write(np.ones((100, 100), dtype=rasterio.uint8), indexes=1)
 
+
 def test_context(tmpdir):
     name = str(tmpdir.join("test_context.tif"))
     with rasterio.open(
@@ -60,13 +74,13 @@ def test_context(tmpdir):
         assert s.height == 100
         assert s.shape == (100, 100)
         assert s.indexes == (1,)
-        assert repr(s) == "<open RasterUpdater name='%s' mode='w'>" % name
+        assert repr(s) == "<open DatasetWriter name='%s' mode='w'>" % name
     assert s.closed
     assert s.count == 1
     assert s.width == 100
     assert s.height == 100
     assert s.shape == (100, 100)
-    assert repr(s) == "<closed RasterUpdater name='%s' mode='w'>" % name
+    assert repr(s) == "<closed DatasetWriter name='%s' mode='w'>" % name
     info = subprocess.check_output(["gdalinfo", name]).decode('utf-8')
     assert "GTiff" in info
     assert "Size is 100, 100" in info
@@ -83,6 +97,8 @@ def test_write_ubyte(tmpdir):
         s.write(a, indexes=1)
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=127.000, Maximum=127.000, Mean=127.000, StdDev=0.000" in info
+
+
 def test_write_ubyte_multi(tmpdir):
     name = str(tmpdir.mkdir("sub").join("test_write_ubyte_multi.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
@@ -93,6 +109,8 @@ def test_write_ubyte_multi(tmpdir):
         s.write(a, 1)
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=127.000, Maximum=127.000, Mean=127.000, StdDev=0.000" in info
+
+
 def test_write_ubyte_multi_list(tmpdir):
     name = str(tmpdir.mkdir("sub").join("test_write_ubyte_multi_list.tif"))
     a = np.array([np.ones((100, 100), dtype=rasterio.ubyte) * 127])
@@ -103,6 +121,8 @@ def test_write_ubyte_multi_list(tmpdir):
         s.write(a, [1])
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=127.000, Maximum=127.000, Mean=127.000, StdDev=0.000" in info
+
+
 def test_write_ubyte_multi_3(tmpdir):
     name = str(tmpdir.mkdir("sub").join("test_write_ubyte_multi_list.tif"))
     arr = np.array(3 * [np.ones((100, 100), dtype=rasterio.ubyte) * 127])
@@ -114,6 +134,7 @@ def test_write_ubyte_multi_3(tmpdir):
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=127.000, Maximum=127.000, Mean=127.000, StdDev=0.000" in info
 
+
 def test_write_float(tmpdir):
     name = str(tmpdir.join("test_write_float.tif"))
     a = np.ones((100, 100), dtype=rasterio.float32) * 42.0
@@ -127,11 +148,13 @@ def test_write_float(tmpdir):
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=42.000, Maximum=42.000, Mean=42.000, StdDev=0.000" in info
 
+
 def test_write_crs_transform(tmpdir):
     name = str(tmpdir.join("test_write_crs_transform.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
-    transform = [101985.0, 300.0379266750948, 0.0,
-                 2826915.0, 0.0, -300.041782729805]
+    transform = affine.Affine(300.0379266750948, 0.0, 101985.0,
+                              0.0, -300.041782729805, 2826915.0)
+
     with rasterio.open(
             name, 'w',
             driver='GTiff', width=100, height=100, count=1,
@@ -145,19 +168,20 @@ def test_write_crs_transform(tmpdir):
     assert 'PROJCS["UTM Zone 18, Northern Hemisphere",' in info
     # make sure that pixel size is nearly the same as transform
     # (precision varies slightly by platform)
-    assert re.search("Pixel Size = \(300.03792\d+,-300.04178\d+\)", info)
+    assert re.search(r'Pixel Size = \(300.03792\d+,-300.04178\d+\)', info)
+
 
 def test_write_crs_transform_affine(tmpdir):
     name = str(tmpdir.join("test_write_crs_transform.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
-    transform = [101985.0, 300.0379266750948, 0.0,
-                 2826915.0, 0.0, -300.041782729805]
+    transform = affine.Affine(300.0379266750948, 0.0, 101985.0,
+                              0.0, -300.041782729805, 2826915.0)
     with rasterio.open(
             name, 'w',
             driver='GTiff', width=100, height=100, count=1,
             crs={'units': 'm', 'no_defs': True, 'ellps': 'WGS84',
                  'proj': 'utm', 'zone': 18},
-            affine=transform,
+            transform=transform,
             dtype=rasterio.ubyte) as s:
         s.write(a, indexes=1)
     assert s.crs.to_dict() == {'init': 'epsg:32618'}
@@ -165,14 +189,15 @@ def test_write_crs_transform_affine(tmpdir):
     assert 'PROJCS["UTM Zone 18, Northern Hemisphere",' in info
     # make sure that pixel size is nearly the same as transform
     # (precision varies slightly by platform)
-    assert re.search("Pixel Size = \(300.03792\d+,-300.04178\d+\)", info)
+    assert re.search(r'Pixel Size = \(300.03792\d+,-300.04178\d+\)', info)
+
 
 def test_write_crs_transform_2(tmpdir):
     """Using 'EPSG:32618' as CRS."""
     name = str(tmpdir.join("test_write_crs_transform.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
-    transform = [101985.0, 300.0379266750948, 0.0,
-                 2826915.0, 0.0, -300.041782729805]
+    transform = affine.Affine(300.0379266750948, 0.0, 101985.0,
+                              0.0, -300.041782729805, 2826915.0)
     with rasterio.open(
             name, 'w',
             driver='GTiff', width=100, height=100, count=1,
@@ -185,14 +210,15 @@ def test_write_crs_transform_2(tmpdir):
     assert 'PROJCS["WGS 84 / UTM zone 18N",' in info
     # make sure that pixel size is nearly the same as transform
     # (precision varies slightly by platform)
-    assert re.search("Pixel Size = \(300.03792\d+,-300.04178\d+\)", info)
+    assert re.search(r'Pixel Size = \(300.03792\d+,-300.04178\d+\)', info)
+
 
 def test_write_crs_transform_3(tmpdir):
     """Using WKT as CRS."""
     name = str(tmpdir.join("test_write_crs_transform.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
-    transform = [101985.0, 300.0379266750948, 0.0,
-                 2826915.0, 0.0, -300.041782729805]
+    transform = affine.Affine(300.0379266750948, 0.0, 101985.0,
+                              0.0, -300.041782729805, 2826915.0)
     wkt = 'PROJCS["UTM Zone 18, Northern Hemisphere",GEOGCS["WGS 84",DATUM["unknown",SPHEROID["WGS84",6378137,298.257223563]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433]],PROJECTION["Transverse_Mercator"],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",-75],PARAMETER["scale_factor",0.9996],PARAMETER["false_easting",500000],PARAMETER["false_northing",0],UNIT["Meter",1]]'
     with rasterio.open(
             name, 'w',
@@ -206,7 +232,8 @@ def test_write_crs_transform_3(tmpdir):
     assert 'PROJCS["UTM Zone 18, Northern Hemisphere",' in info
     # make sure that pixel size is nearly the same as transform
     # (precision varies slightly by platform)
-    assert re.search("Pixel Size = \(300.03792\d+,-300.04178\d+\)", info)
+    assert re.search(r'Pixel Size = \(300.03792\d+,-300.04178\d+\)', info)
+
 
 def test_write_meta(tmpdir):
     name = str(tmpdir.join("test_write_meta.tif"))
@@ -217,6 +244,7 @@ def test_write_meta(tmpdir):
     info = subprocess.check_output(["gdalinfo", "-stats", name]).decode('utf-8')
     assert "Minimum=127.000, Maximum=127.000, Mean=127.000, StdDev=0.000" in info
 
+
 def test_write_nodata(tmpdir):
     name = str(tmpdir.join("test_write_nodata.tif"))
     a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
@@ -240,20 +268,6 @@ def test_guard_nodata(tmpdir):
             dtype=a.dtype, nodata=-1)
 
 
-def test_write_lzw(tmpdir):
-    name = str(tmpdir.join("test_write_lzw.tif"))
-    a = np.ones((100, 100), dtype=rasterio.ubyte) * 127
-    with rasterio.open(
-            name, 'w',
-            driver='GTiff',
-            width=100, height=100, count=1,
-            dtype=a.dtype,
-            compress='LZW') as s:
-        assert ('compress', 'LZW') in s.kwds.items()
-        s.write(a, indexes=1)
-    info = subprocess.check_output(["gdalinfo", name]).decode('utf-8')
-    assert "LZW" in info
-
 def test_write_noncontiguous(tmpdir):
     name = str(tmpdir.join("test_write_nodata.tif"))
     ROWS = 4
@@ -276,11 +290,33 @@ def test_write_noncontiguous(tmpdir):
             dst.write(arr[:, :, i], indexes=i + 1)
 
 
-@pytest.mark.parametrize("driver", BAD_WRITE_DRIVERS)
+@pytest.mark.parametrize("driver", list(blacklist.keys()))
 def test_write_blacklist(tmpdir, driver):
+
+    # Skip if we don't have driver support built in.
+    with Env() as env:
+        if driver not in env.drivers():
+            pytest.skip()
+
     name = str(tmpdir.join("data.test"))
     with pytest.raises(RasterioIOError) as exc_info:
         rasterio.open(name, 'w', driver=driver, width=100, height=100,
                       count=1, dtype='uint8')
     exc = str(exc_info.value)
-    assert "Rasterio does not support writing with {} driver".format(driver) in exc
+    assert exc.startswith("Blacklisted")
+
+
+def test_creation_metadata_deprecation(tmpdir):
+    name = str(tmpdir.join("test.tif"))
+    with rasterio.open(name, 'w', driver='GTiff', height=1, width=1, count=1, dtype='uint8', BIGTIFF=True) as dst:
+        dst.write(np.ones((1, 1, 1), dtype='uint8'))
+        assert dst.tags(ns='rio_creation_kwds') == {}
+
+
+def test_wplus_transform(tmpdir):
+    """Transform is set on a new dataset created in w+ mode (see issue #1359)"""
+    name = str(tmpdir.join("test.tif"))
+    transform = affine.Affine.translation(10.0, 10.0) * affine.Affine.scale(0.5, -0.5)
+    with rasterio.open(name, 'w+', driver='GTiff', crs='epsg:4326', transform=transform, height=10, width=10, count=1, dtype='uint8') as dst:
+        dst.write(np.ones((1, 10, 10), dtype='uint8'))
+        assert dst.transform == transform
